%{
/* cfgtoken.l -- Tokenize the configuration file.
   $Id: cfgtoken.l,v 0.4 1999/04/22 09:05:03 eagle Exp $

   Written by Russ Allbery <rra@stanford.edu>
   Copyright 1998, 1999 Board of Trustees, Leland Stanford Jr. University

   This program is free software.  See README included in the distribution
   for terms and conditions.

   Tokenize the configuration file.  This is a fairly smart lexer, since we
   want to give it sufficient intelligence to figure out when to possibly
   expect a number and do the conversion here.  (That saves us lots of
   trouble at the yacc stage of things.)  Note that we're not *quite*
   intelligent enough to be able to handle all-numeric log names, but we do
   allow quoted strings to coerce matters. */

#include "cfgparse.h"           /* Token defines and yacc interface */
#include "utility.h"            /* xmalloc() */

#include <string.h>             /* strcpy(), strncpy() */

/* cfgdie() is defined in cfgparse.y.  Lexers and parsers are incestuous. */
extern void cfgdie (char *format, ...);
  
/* The current line number of the file.  We keep track of this ourselves
   since although lex maintains yylineno, flex doesn't by default, and it's
   easy for us.  Expose it to the parser as well. */
int cfgline = 1;
%}

%start  S_ARGS S_BLOCK S_COMMAND S_SET S_VAL

LETTER  [a-zA-Z]
DIGIT   [0-9]
NAME    {LETTER}({LETTER}|{DIGIT}|_)*
WORD    [^ \t\n]+
NUMBER  {DIGIT}+
STRING  \"[^\"]*\"

%%

[ \t]+                  /* Ignore whitespace except for newlines. */

<INITIAL,S_BLOCK>#.*\n  cfgline++;

<INITIAL>set            BEGIN S_SET; return SET;

<S_SET>{NAME}           {
                            yylval.string = xmalloc (yyleng + 1);
                            strcpy (yylval.string, yytext);
                            return VARIABLE;
                        }

<S_SET>=                BEGIN S_VAL; return EQ;

<INITIAL>{NAME}         {
                            BEGIN S_BLOCK;
                            yylval.string = xmalloc (yyleng + 1);
                            strcpy (yylval.string, yytext);
                            return LABEL;
                        }

<S_BLOCK>\{             return LBRACE;
<S_BLOCK>\}             BEGIN INITIAL; return RBRACE;

<S_BLOCK>archive[ \t]*: BEGIN S_ARGS; return ARCHIVE;
<S_BLOCK>copy[ \t]*:    BEGIN S_ARGS; return COPY;
<S_BLOCK>log[ \t]*:     BEGIN S_ARGS; return LOG;
<S_BLOCK>restart[ \t]*: BEGIN S_COMMAND; return RESTART;
<S_BLOCK>stop[ \t]*:    BEGIN S_COMMAND; return STOP;

<S_COMMAND>run          BEGIN S_ARGS; return RUN;
<S_COMMAND>hup          BEGIN S_ARGS; return HUP;

<S_ARGS>{NUMBER}        {
                            yylval.number = atoi (yytext);
                            return NUMBER;
                        }

<S_ARGS,S_VAL>{WORD}    {
                            yylval.string = xmalloc (yyleng + 1);
                            strcpy (yylval.string, yytext);
                            return STRING;
                        }

<S_ARGS,S_VAL>{STRING}  {
                            yylval.string = xmalloc (yyleng - 1);
                            strncpy (yylval.string, yytext + 1, yyleng - 2);
                            yylval.string[yyleng - 2] = '\0';
                            return STRING;
                        }

<S_ARGS>\n              BEGIN S_BLOCK; cfgline++; return EOL;
<S_VAL>\n               BEGIN INITIAL; cfgline++; return EOL;
<INITIAL,S_BLOCK>\n     cfgline++;

\n                      cfgdie ("Unexpected end of line");
.                       cfgdie ("Syntax error at or near `%c'", yytext[0]);

%%

/* Ensure that we exit at the end of the file. */
int yywrap () { return 1; }
